"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useTreeData;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _vue = require("vue");

var _legacyUtil = require("../utils/legacyUtil");

function parseSimpleTreeData(treeData, _ref) {
  var id = _ref.id,
      pId = _ref.pId,
      rootPId = _ref.rootPId;
  var keyNodes = {};
  var rootNodeList = []; // Fill in the map

  var nodeList = treeData.map(function (node) {
    var clone = (0, _objectSpread2.default)({}, node);
    var key = clone[id];
    keyNodes[key] = clone;
    clone.key = clone.key || key;
    return clone;
  }); // Connect tree

  nodeList.forEach(function (node) {
    var parentKey = node[pId];
    var parent = keyNodes[parentKey]; // Fill parent

    if (parent) {
      parent.children = parent.children || [];
      parent.children.push(node);
    } // Fill root tree node


    if (parentKey === rootPId || !parent && rootPId === null) {
      rootNodeList.push(node);
    }
  });
  return rootNodeList;
}
/**
 * Convert `treeData` or `children` into formatted `treeData`.
 * Will not re-calculate if `treeData` or `children` not change.
 */


function useTreeData(treeData, children, simpleMode) {
  var mergedTreeData = (0, _vue.shallowRef)();
  (0, _vue.watch)([simpleMode, treeData, children], function () {
    var simpleModeValue = simpleMode.value;

    if (treeData.value) {
      mergedTreeData.value = simpleMode.value ? parseSimpleTreeData((0, _vue.toRaw)(treeData.value), (0, _objectSpread2.default)({
        id: 'id',
        pId: 'pId',
        rootPId: null
      }, simpleModeValue !== true ? simpleModeValue : {})) : (0, _vue.toRaw)(treeData.value).slice();
    } else {
      mergedTreeData.value = (0, _legacyUtil.convertChildrenToData)((0, _vue.toRaw)(children.value));
    }
  }, {
    immediate: true,
    deep: true
  });
  return mergedTreeData;
}